@@ -1,6 +1,6 @@
 /* GNU Datamash - perform simple calculation on input data
 
-   Copyright (C) 2013-2015 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2013-2016 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -34,6 +34,7 @@
 
 #include "system.h"
 
+#include "fpucw.h"
 #include "closeout.h"
 #include "hash.h"
 #include "hash-pjw.h"
@@ -48,12 +49,14 @@
 #include "xstrndup.h"
 #include "xalloc.h"
 
-#include "strcnt.h"
 #include "text-options.h"
 #include "text-lines.h"
 #include "column-headers.h"
+#include "op-defs.h"
+#include "op-parser.h"
 #include "field-ops.h"
 #include "utils.h"
+#include "crosstab.h"
 
 /* The official name of this program (e.g., no 'g' prefix).  */
 #define PROGRAM_NAME "datamash"
@@ -78,35 +81,20 @@ static bool input_header = false;
 /* If true, print the entire input line. Otherwise, print only the key fields */
 static bool print_full_line = false;
 
-struct group_column_t
-{
-  size_t col_number;    /* 1 = first field */
-  bool   col_by_name;   /* true if the user gave a column name */
-  char*  col_name;      /* column name - to be converted to number after
-                           header line is read */
-};
-static bool   group_column_have_names = false;
-static size_t num_group_columns = 0;
-static struct group_column_t *group_columns = NULL;
+/* processing mode, group fields, field operations */
+static struct datamash_ops *dm = NULL;
 
 static bool line_mode = false; /* if TRUE, handle each line as a group */
 
+static bool crosstab_mode = false; /* if TRUE, do cross-tabulation */
+
+static struct crosstab* crosstab = NULL;
+
 static bool pipe_through_sort = false;
 static FILE* input_stream = NULL;
 
-/* if true, 'transpose' and 'reverse' require every line to have
-   the exact same number of fields. Otherwise, the program
-   will fail with non-zero exit code. */
-static bool strict = true;
-
-/* if 'strict' is false, lines with fewer-than-expected fields
-   will be filled with this value */
-static char* missing_field_filler = "N/A";
-
-/* If true, N/A, NaN and empty cells in numeric operations will be silently
-   ignored instead of triggering an error. Akin to R's "na.rm=TRUE" option. */
-bool remove_na_values = false;
-
+/* Use large buffer for normal operation (will be reduced for testing) */
+static size_t rmdup_initial_size = (1024*1024);
 
 enum
 {
@@ -116,7 +104,8 @@ enum
   REMOVE_NA_VALUES_OPTION,
   UNDOC_PRINT_INF_OPTION,
   UNDOC_PRINT_NAN_OPTION,
-  UNDOC_PRINT_PROGNAME_OPTION
+  UNDOC_PRINT_PROGNAME_OPTION,
+  UNDOC_RMDUP_TEST
 };
 
 static char const short_options[] = "sfF:izg:t:HW";
@@ -142,25 +131,26 @@ static struct option const long_options[] =
   {"-print-inf", no_argument, NULL, UNDOC_PRINT_INF_OPTION},
   {"-print-nan", no_argument, NULL, UNDOC_PRINT_NAN_OPTION},
   {"-print-progname", no_argument, NULL, UNDOC_PRINT_PROGNAME_OPTION},
+  {"-rmdup-test", no_argument, NULL, UNDOC_RMDUP_TEST},
   {NULL, 0, NULL, 0},
 };
 
 void
 group_columns_find_named_columns ()
 {
-  for (size_t i = 0; i < num_group_columns; ++i)
+  for (size_t i = 0; i < dm->num_grps; ++i)
     {
-      struct group_column_t *p = &group_columns[i];
+      struct group_column_t *p = &dm->grps[i];
 
-      if (!p->col_by_name)
+      if (!p->by_name)
         continue;
 
-      p->col_number = get_input_field_number (p->col_name);
-      if (p->col_number == 0)
+      p->num = get_input_field_number (p->name);
+      if (p->num == 0)
         error (EXIT_FAILURE, 0,
                 _("column name %s not found in input file"),
-                quote (p->col_name));
-      p->col_by_name = false;
+                quote (p->name));
+      p->by_name = false;
     }
 }
 
@@ -171,27 +161,32 @@ usage (int status)
     emit_try_help ();
   else
     {
-      printf (_("Usage: %s [OPTION] op [col] [op col ...]\n"),
+      printf (_("Usage: %s [OPTION] op [fld] [op fld ...]\n"),
           program_name);
       fputs ("\n", stdout);
       fputs (_("Performs numeric/string operations on input from stdin."),
           stdout);
       fputs ("\n\n", stdout);
-      fputs (_("'op' is the operation to perform;\n"), stdout);
       fputs (_("\
-For grouping,per-line operations 'col' is the input field to use;\n\
-'col' can be a number (1=first field), or a column name when using\n\
--H or --header-in options.\n\
-"), stdout);
+'op' is the operation to perform.  If a primary operation is used,\n\
+it must be listed first, optionally followed by other operations.\n"), stdout);
+      fputs (_("\
+'fld' is the input field to use.  'fld' can be a number (1=first field),\n\
+or a field name when using the -H or --header-in options.\n"), stdout);
+      fputs (_("\
+Multiple fields can be listed with a comma (e.g. 1,6,8).  A range of\n\
+fields can be listed with a dash (e.g. 2-8).  Use colons for operations\n\
+which require a pair of fields (e.g. 'pcov 2:6').\n"), stdout);
       fputs ("\n\n", stdout);
-      fputs (_("File operations:\n"),stdout);
-      fputs ("  transpose, reverse\n",stdout);
+      fputs (_("Primary operations:\n"),stdout);
+      fputs ("  groupby, crosstab, transpose, reverse, check\n",stdout);
 
       fputs (_("Line-Filtering operations:\n"),stdout);
       fputs ("  rmdup\n",stdout);
 
       fputs (_("Per-Line operations:\n"),stdout);
-      fputs ("  base64, debase64, md5, sha1, sha256, sha512\n",stdout);
+      fputs ("  base64, debase64, md5, sha1, sha256, sha512,\n", stdout);
+      fputs ("  bin, strbin, round, floor, ceil, trunc, frac\n", stdout);
 
       fputs (_("Numeric Grouping operations:\n"),stdout);
       fputs ("  sum, min, max, absmin, absmax\n",stdout);
@@ -202,19 +197,20 @@ For grouping,per-line operations 'col' is the input field to use;\n\
 
       fputs (_("Statistical Grouping operations:\n"),stdout);
       fputs ("\
-  mean, median, q1, q3, iqr, mode, antimode, pstdev, sstdev, pvar\n\
-  svar, mad, madraw, pskew, sskew, pkurt, skurt, dpo, jarque\n\
+  mean, median, q1, q3, iqr, mode, antimode, pstdev, sstdev, pvar,\n\
+  svar, mad, madraw, pskew, sskew, pkurt, skurt, dpo, jarque,\n\
+  scov, pcov, spearson, ppearson\n\
 \n", stdout);
       fputs ("\n", stdout);
 
-      fputs (_("Options:\n"),stdout);
       fputs (_("Grouping Options:\n"),stdout);
       fputs (_("\
   -f, --full                print entire input line before op results\n\
                               (default: print only the grouped keys)\n\
 "), stdout);
       fputs (_("\
-  -g, --group=X[,Y,Z]       group via fields X,[Y,Z]\n\
+  -g, --group=X[,Y,Z]       group via fields X,[Y,Z];\n\
+                              equivalent to primary operation 'groupby'\n\
 "), stdout);
       fputs (_("\
       --header-in           first input line is column headers\n\
@@ -278,7 +274,7 @@ For grouping,per-line operations 'col' is the input field to use;\n\
 \n", program_name);
 
       fputs (_("For detailed usage information and examples, see\n"),stdout);
-      printf ("  man %s\n", program_name);
+      printf ("  man %s\n", PACKAGE_NAME);
       fputs (_("The manual and more examples are available at\n"), stdout);
       fputs ("  " PACKAGE_URL "\n\n", stdout);
     }
@@ -303,15 +299,27 @@ safe_line_record_get_field (const struct line_record_t *lr, const size_t n,
     error_not_enough_fields (n, line_record_num_fields (lr));
 }
 
+static inline void
+safe_line_record_get_fieldz (const struct line_record_t *lr, const size_t n,
+                             char * /* out */ buf, size_t len)
+{
+  const char *p;
+  size_t l;
+  safe_line_record_get_field (lr, n, &p, &l);
+  l = MIN (len-1,l);
+  memcpy (buf, p, l);
+  buf[l] = 0;
+}
+
 /* returns TRUE if the lines are different, false if identical.
  * comparison is based on the specified keys */
 /* copied from coreutils's src/uniq.c (in the key-spec branch) */
 static bool
 different (const struct line_record_t* l1, const struct line_record_t* l2)
 {
-  for (size_t i = 0; i < num_group_columns; ++i)
+  for (size_t i = 0; i < dm->num_grps; ++i)
     {
-      const size_t col_num = group_columns[i].col_number;
+      const size_t col_num = dm->grps[i].num;
       const char *str1=NULL,*str2=NULL;
       size_t len1=0,len2=0;
       safe_line_record_get_field (l1, col_num, &str1, &len1);
@@ -335,9 +343,9 @@ process_line (const struct line_record_t *line)
   enum FIELD_OP_COLLECT_RESULT flocr;
   bool keep_line = false;
 
-  struct fieldop *op = field_ops;
-  while (op)
+  for (size_t i=0; i<dm->num_ops; ++i)
     {
+      struct fieldop *op = &dm->ops[i];
       safe_line_record_get_field (line, op->field, &str, &len);
       flocr = field_op_collect (op, str, len);
       if (!field_op_ok (flocr))
@@ -351,8 +359,6 @@ process_line (const struct line_record_t *line)
               (uintmax_t)line_number, (uintmax_t)op->field, tmp);
         }
       keep_line = keep_line || (flocr==FLOCR_OK_KEEP_LINE);
-
-      op = op->next;
     }
   return keep_line;
 }
@@ -377,9 +383,9 @@ print_input_line (const struct line_record_t* lb)
     }
   else
     {
-      for (size_t i = 0; i < num_group_columns; ++i)
+      for (size_t i = 0; i < dm->num_grps; ++i)
         {
-          const size_t col_num = group_columns[i].col_number;
+          const size_t col_num = dm->grps[i].num;
           safe_line_record_get_field (lb, col_num, &str, &len);
           ignore_value (fwrite (str,sizeof (char),len,stdout));
           print_field_separator ();
@@ -403,6 +409,7 @@ print_column_headers ()
 {
   if (print_full_line)
     {
+      /* Print the headers of all the input fields */
       for (size_t n=1; n<=get_num_column_headers (); ++n)
         {
           fputs (get_input_field_name (n), stdout);
@@ -411,9 +418,12 @@ print_column_headers ()
     }
   else
     {
-      for (size_t i = 0; i < num_group_columns; ++i)
+      /* print only the headers of the group-by fields, e.g
+        'GroupBy (field-3)'  (without input headers), or
+        'GroupBy (NAME)'     (with input headers)           */
+      for (size_t i = 0; i < dm->num_grps; ++i)
         {
-          const size_t col_num = group_columns[i].col_number;
+          const size_t col_num = dm->grps[i].num;
           if (col_num > get_num_column_headers ())
             error_not_enough_fields (col_num, get_num_column_headers ());
           printf ("GroupBy" "(%s)",get_input_field_name (col_num));
@@ -421,13 +431,21 @@ print_column_headers ()
         }
     }
 
-  for (struct fieldop *op = field_ops; op ; op=op->next)
+  /* add headers of the operations, e.g.
+        'sum (field-3)'  (without input headers), or
+        'sum (NAME)'     (with input headers)           */
+  for (size_t i=0; i<dm->num_ops; ++i)
     {
+      struct fieldop *op = &dm->ops[i];
+      if (op->slave)
+        continue;
+
       if (op->field > get_num_column_headers ())
         error_not_enough_fields (op->field, get_num_column_headers ());
-      printf ("%s" "(%s)",op->name, get_input_field_name (op->field));
+      printf ("%s" "(%s)",get_field_operation_name (op->op),
+                          get_input_field_name (op->field));
 
-      if (op->next)
+      if (i != dm->num_ops-1)
         print_field_separator ();
     }
 
@@ -435,6 +453,23 @@ print_column_headers ()
   print_line_separator ();
 }
 
+void
+field_op_find_named_columns ()
+{
+  for (size_t i=0; i<dm->num_ops; ++i)
+    {
+      struct fieldop *p = &dm->ops[i];
+      if (!p->field_by_name)
+        continue;
+      p->field = get_input_field_number (p->field_name);
+      if (p->field == 0)
+        error (EXIT_FAILURE, 0,
+                _("column name %s not found in input file"),
+                quote (p->field_name));
+      p->field_by_name = false;
+    }
+}
+
 /*
     Process the input header line
 */
@@ -450,14 +485,78 @@ process_input_header (FILE *stream)
       line_number++;
       /* If using named-columns, find the column numbers after reading the
          header line. */
-      if (field_op_have_named_fields ())
-        field_op_find_named_columns ();
-      if (group_column_have_names)
-        group_columns_find_named_columns ();
+      field_op_find_named_columns ();
+      group_columns_find_named_columns ();
     }
   line_record_free (&lr);
 }
 
+void
+summarize_field_ops ()
+{
+  for (size_t i=0;i<dm->num_ops;++i)
+    {
+      struct fieldop *p = &dm->ops[i];
+      if (p->slave)
+        continue;
+
+      field_op_summarize (p);
+      fputs (p->out_buf, stdout);
+
+      /* print field separator */
+      if (i != dm->num_ops-1)
+        print_field_separator ();
+    }
+
+  /* print end-of-line */
+  print_line_separator ();
+}
+
+void
+reset_field_ops ()
+{
+  for (size_t i=0;i<dm->num_ops;++i)
+    field_op_reset (&dm->ops[i]);
+}
+
+/* Process a completed group of data lines
+   (all with the same 'group by' keys). */
+static void
+process_group (const struct line_record_t* line)
+{
+  /* TODO: dynamically re-alloc if needed */
+  char col_name[512];
+  char row_name[512];
+
+  if (lines_in_group>0)
+    {
+      if (crosstab_mode)
+        {
+          /* cross-tabulation mode - save results in a matrix, print later */
+          const size_t row_field = dm->grps[0].num;
+          safe_line_record_get_fieldz (line, row_field,
+                                       row_name, sizeof row_name);
+
+          const size_t col_field = dm->grps[1].num;
+          safe_line_record_get_fieldz (line, col_field,
+                                       col_name, sizeof col_name);
+
+          field_op_summarize (&dm->ops[0]);
+          const char* data = dm->ops[0].out_buf;
+
+          crosstab_add_result (crosstab, row_name, col_name, data);
+        }
+      else
+        {
+          /* group-by/per-line mode - print results once available */
+          print_input_line (line);
+          summarize_field_ops ();
+        }
+    }
+  lines_in_group = 0;
+  reset_field_ops ();
+}
+
 /*
     Process each line in the input.
 
@@ -480,11 +579,7 @@ process_file ()
      in 'open_input' - read it now */
   if (input_header && line_number==0)
     process_input_header (input_stream);
-  /* If using named-columns, but no input header - abort */
-  if ((field_op_have_named_fields () || group_column_have_names)
-      && !input_header)
-    error (EXIT_FAILURE, 0,
-           _("-H or --header-in must be used with named columns"));
+
   /* If there is an input header line, and the user requested an output
      header line, and the input line was read successfully, print headers */
   if (input_header && output_header && line_number==1)
@@ -510,17 +605,14 @@ process_file ()
 
 
       /* If no keys are given, the entire input is considered one group */
-      if (num_group_columns || line_mode)
+      if (dm->num_grps || line_mode)
         {
           new_group = (group_first_line->lbuf.length == 0 || line_mode
                        || different (thisline, group_first_line));
 
-          if (new_group && lines_in_group>0)
+          if (new_group)
             {
-              print_input_line (group_first_line);
-              summarize_field_ops ();
-              reset_field_ops ();
-              lines_in_group = 0 ;
+              process_group (group_first_line);
               group_first_line->lbuf.length = 0;
             }
         }
@@ -540,12 +632,7 @@ process_file ()
     }
 
   /* summarize last group */
-  if (lines_in_group)
-    {
-      print_input_line (group_first_line);
-      summarize_field_ops ();
-      reset_field_ops ();
-    }
+  process_group (group_first_line);
 
   line_record_free (&lb1);
   line_record_free (&lb2);
@@ -621,7 +708,7 @@ transpose_file ()
 }
 
 /*
-    Revers the fields in each line of the input file
+    Reverse the fields in each line of the input file
  */
 static void
 reverse_fields_in_file ()
@@ -718,6 +805,68 @@ noop_file ()
   line_record_free (&lr);
 }
 
+/*
+   Read file, ensure it is in tabular format
+   (same number of fields in each line)
+ */
+static void
+tabular_check_file ()
+{
+  size_t prev_num_fields=0;
+  struct line_record_t lb1, lb2;
+  struct line_record_t *thisline, *prevline;
+
+  thisline = &lb1;
+  prevline = &lb2;
+
+  line_record_init (thisline);
+  line_record_init (prevline);
+
+  while (true)
+    {
+      if (!line_record_fread (thisline, input_stream, eolchar))
+        break;
+      line_number++;
+
+      const size_t num_fields = line_record_num_fields (thisline);
+
+      if (line_number>1 && num_fields != prev_num_fields)
+        {
+          fprintf (stderr, _("line %"PRIuMAX" (%"PRIuMAX" fields):\n  "),
+                       (uintmax_t)(line_number-1), (uintmax_t)prev_num_fields);
+          ignore_value (fwrite (line_record_buffer (prevline),
+                                line_record_length (prevline), sizeof (char),
+                                stderr));
+          fputc ('\n', stderr);
+          fprintf (stderr, _("line %"PRIuMAX" (%"PRIuMAX" fields):\n  "),
+                       (uintmax_t)(line_number), (uintmax_t)num_fields);
+          ignore_value (fwrite (line_record_buffer (thisline),
+                                line_record_length (thisline), sizeof (char),
+                                stderr));
+          fputc ('\n', stderr);
+          error (EXIT_FAILURE, 0, _("check failed: line " \
+                       "%"PRIuMAX" has %"PRIuMAX" fields (previous line had "\
+                       "%"PRIuMAX")"),
+                       (uintmax_t)line_number, (uintmax_t)num_fields,
+                       (uintmax_t)prev_num_fields);
+        }
+      prev_num_fields = num_fields;
+
+      SWAP_LINES (prevline, thisline);
+    }
+
+  /* Print summary */
+  printf (ngettext ("%"PRIuMAX" line", "%"PRIuMAX" lines",
+          select_plural (line_number)), (uintmax_t)line_number);
+  fputs (", ", stdout);
+  printf (ngettext ("%"PRIuMAX" field", "%"PRIuMAX" fields",
+          select_plural (prev_num_fields)), (uintmax_t)prev_num_fields);
+  print_line_separator ();
+
+  line_record_free (&lb1);
+  line_record_free (&lb2);
+}
+
 /*
     Remove lines with duplicates keys from a file
  */
@@ -729,7 +878,7 @@ remove_dups_in_file ()
   struct line_record_t lr;
   struct line_record_t *thisline;
   Hash_table *ht;
-  const size_t init_table_size = 100000;
+  const size_t init_table_size = rmdup_initial_size;
 
   char* keys_buffer = NULL;
   size_t keys_buffer_alloc = 0;
@@ -739,20 +888,17 @@ remove_dups_in_file ()
   size_t buffer_list_alloc = 0;
   size_t buffer_list_size = 0;
 
-  assert (field_ops != NULL); /* LCOV_EXCL_LINE */
-
   thisline = &lr;
   line_record_init (thisline);
   ht = hash_initialize (init_table_size, NULL,
                         hash_pjw, hash_compare_strings, NULL);
 
   /* Allocate keys buffer */
-  keys_buffer_alloc = 1000000 ;
+  keys_buffer_alloc = rmdup_initial_size ;
   keys_buffer = xmalloc ( keys_buffer_alloc );
 
   /* List of allocated key-buffers */
-  buffer_list_alloc = 1000 ;
-  buffer_list = xnmalloc (buffer_list_alloc, sizeof (char*));
+  buffer_list = x2nrealloc (NULL, &buffer_list_alloc, sizeof (char*));
   buffer_list[0] = keys_buffer;
   buffer_list_size = 1 ;
 
@@ -764,10 +910,10 @@ remove_dups_in_file ()
 
           /* If using named-columns, find the column numbers after reading the
              header line. */
-          if (field_op_have_named_fields ())
+          if (dm->header_required)
             {
               build_input_line_headers (&lr, true);
-              field_op_find_named_columns ();
+              group_columns_find_named_columns ();
             }
 
           if (output_header)
@@ -779,10 +925,9 @@ remove_dups_in_file ()
             }
         }
     }
-  if (!input_header && field_op_have_named_fields ())
-    error (EXIT_FAILURE, 0,
-           _("-H or --header-in must be used with named columns"));
-  const size_t key_col = field_ops->field;
+
+  assert (dm->num_grps==1); /* LCOV_EXCL_LINE */
+  const size_t key_col = dm->grps[0].num;
 
   /* TODO: handle (output_header && !input_header) by generating dummy headers
            after the first line is read, and the number of fields is known. */
@@ -801,13 +946,11 @@ remove_dups_in_file ()
          {
            keys_buffer = xmalloc ( keys_buffer_alloc ) ;
            next_key_pos = 0;
+
            /* Add new key-buffer to the list */
-           if (buffer_list_size >= buffer_list_alloc)
-             {
-               buffer_list_alloc += 1000 ;
-               buffer_list = xnrealloc (buffer_list, buffer_list_alloc,
-                                        sizeof (char*));
-             }
+           if (buffer_list_size == buffer_list_alloc)
+              buffer_list = x2nrealloc (buffer_list,
+                                        &buffer_list_alloc, sizeof (char*));
            buffer_list[buffer_list_size++] = keys_buffer;
          }
        char *next_key = keys_buffer+next_key_pos;
@@ -840,7 +983,7 @@ remove_dups_in_file ()
 static void
 open_input ()
 {
-  if (pipe_through_sort && num_group_columns>0)
+  if (pipe_through_sort && dm->num_grps>0)
     {
       char tmp[INT_BUFSIZE_BOUND (size_t)*2+5];
       char cmd[1024];
@@ -872,9 +1015,9 @@ open_input ()
           snprintf (tmp,sizeof (tmp),"-t %c%c%c ",qc,in_tab,qc);
           strcat (cmd,tmp);
         }
-      for (size_t i = 0; i < num_group_columns; ++i)
+      for (size_t i = 0; i < dm->num_grps; ++i)
         {
-          const size_t col_num = group_columns[i].col_number;
+          const size_t col_num = dm->grps[i].num;
           snprintf (tmp,sizeof (tmp),"-k%"PRIuMAX",%"PRIuMAX" ",
                     (uintmax_t)col_num,(uintmax_t)col_num);
           if (strlen (tmp)+strlen (cmd)+1 >= sizeof (cmd))
@@ -912,59 +1055,15 @@ close_input ()
     error (EXIT_FAILURE, errno, _("read error (on close)"));
 }
 
-static void
-free_sort_keys ()
-{
-  free (group_columns);
-}
-
-/* Parse the "--group=X[,Y,Z]" parameter, populating 'keylist' */
-static void
-parse_group_spec ( char* spec )
-{
-  long int val;
-  size_t idx;
-  char *tok, *endptr;
-
-  /* Count number of groups parameters, by number of commas */
-  num_group_columns = strcnt (spec, ',')+1;
-
-  /* Allocate the group_columns */
-  group_columns = xnmalloc (num_group_columns, sizeof (struct group_column_t));
-
-  idx=0;
-  while ( (tok = strsep (&spec, ",")) != NULL)
-    {
-      if (strlen (tok) == 0)
-        error (EXIT_FAILURE, 0, _("invalid empty grouping parameter"));
-      errno = 0 ;
-      val = strtol (tok, &endptr, 10);
-      if (errno == 0 && endptr != tok && *endptr == 0)
-        {
-          /* If strtol succeeded, it's a valid number. */
-          if (val<1)
-            error (EXIT_FAILURE, 0, _("invalid grouping parameter %s"),
-                                        quote (tok));
-          group_columns[idx].col_number = val;
-          group_columns[idx].col_by_name = false;
-          group_columns[idx].col_name = NULL;
-        }
-      else
-        {
-          /* If strrol failed, assume it's a named column - resolve it later. */
-          group_columns[idx].col_name = xstrdup (tok);
-          group_columns[idx].col_number = 0;
-          group_columns[idx].col_by_name = true;
-          group_column_have_names = true;
-        }
-      idx++;
-    }
-}
-
 int main (int argc, char* argv[])
 {
   int optc;
-  enum operation_mode op_mode;
+  enum processing_mode premode = MODE_INVALID;
+  const char* premode_group_spec = NULL;
+
+  DECL_LONG_DOUBLE_ROUNDING
+  BEGIN_LONG_DOUBLE_ROUNDING ();
+
   set_program_name (argv[0]);
 
   setlocale (LC_ALL, "");
@@ -990,7 +1089,8 @@ int main (int argc, char* argv[])
           break;
 
         case 'g':
-          parse_group_spec (optarg);
+          premode = MODE_GROUPBY;
+          premode_group_spec = optarg;
           break;
 
         case 'i':
@@ -1039,12 +1139,8 @@ int main (int argc, char* argv[])
 
         case UNDOC_PRINT_INF_OPTION:
         case UNDOC_PRINT_NAN_OPTION:
-          {
-            struct fieldop op;
-            op.op = (optc==UNDOC_PRINT_INF_OPTION)?OP_MAX:OP_MEAN;
-            op.res_type = NUMERIC_RESULT;
-            field_op_summarize_empty (&op);
-          }
+          field_op_print_empty_value ( (optc==UNDOC_PRINT_INF_OPTION)
+                                       ?OP_MAX:OP_MEAN );
           exit (EXIT_SUCCESS);
           break;
 
@@ -1053,6 +1149,10 @@ int main (int argc, char* argv[])
           exit (EXIT_SUCCESS);
           break;
 
+        case UNDOC_RMDUP_TEST:
+          rmdup_initial_size = 1024;
+          break;
+
         case_GETOPT_HELP_CHAR;
 
         case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
@@ -1062,48 +1162,74 @@ int main (int argc, char* argv[])
         }
     }
 
+
   if (argc <= optind)
     {
       error (0, 0, _("missing operation specifiers"));
       usage (EXIT_FAILURE);
     }
 
-  op_mode = parse_operation_mode (argc, optind, argv);
+  /* The rest of the parameters are the operations */
+  if (premode == MODE_INVALID)
+    dm = datamash_ops_parse (argc - optind, (const char**)argv+optind);
+  else
+    dm = datamash_ops_parse_premode (premode, premode_group_spec,
+                            argc - optind, (const char**)argv+optind);
+
+  /* If using named-columns, but no input header - abort */
+  if (dm->header_required && !input_header)
+    error (EXIT_FAILURE, 0,
+           _("-H or --header-in must be used with named columns"));
 
   open_input ();
-  switch (op_mode)
+  switch (dm->mode)                              /* LCOV_EXCL_BR_LINE */
     {
-    case LINE_MODE:
+    case MODE_PER_LINE:
       line_mode = true;
       /* fall through */
-    case GROUPING_MODE:
+    case MODE_GROUPBY:
       process_file ();
       break;
 
-    case NOOP_MODE:
+    case MODE_NOOP:
       noop_file ();
       break;
 
-    case TRANSPOSE_MODE:
+    case MODE_TRANSPOSE:
       transpose_file ();
       break;
 
-    case REVERSE_FIELD_MODE:
+    case MODE_REVERSE:
       reverse_fields_in_file ();
       break;
 
-    case REMOVE_DUPS_MODE:
+    case MODE_REMOVE_DUPS:
       remove_dups_in_file ();
       break;
 
-    case UNKNOWN_MODE:
+    case MODE_CROSSTAB:
+      assert ( dm->num_grps== 2 ); /* LCOV_EXCL_LINE */
+      assert ( dm->num_ops == 1 ); /* LCOV_EXCL_LINE */
+      crosstab_mode = true;
+      crosstab = crosstab_init ();
+      process_file ();
+      crosstab_print (crosstab);
+      crosstab_free (crosstab);
+      break;
+
+    case MODE_TABULAR_CHECK:
+      tabular_check_file ();
+      break;
+
+    case MODE_INVALID:
     default:
-      internal_error ("op mode"); /* LCOV_EXCL_LINE */
+      internal_error ("op mode");                /* LCOV_EXCL_LINE */
     }
-  free_field_ops ();
-  free_sort_keys ();
   free_column_headers ();
   close_input ();
+  datamash_ops_free (dm);
+
+  END_LONG_DOUBLE_ROUNDING ();
 
   return EXIT_SUCCESS;
 }
