@@ -1,6 +1,6 @@
 /* GNU Datamash - perform simple calculation on input data
 
-   Copyright (C) 2013-2015 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2013-2016 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -26,6 +26,8 @@
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdint.h>
+#include <inttypes.h>
 #include <string.h>
 #include <stdbool.h>
 #include <time.h>
@@ -33,7 +35,6 @@
 #include "error.h"
 #include "minmax.h"
 #include "linebuffer.h"
-#include "quote.h"
 #include "system.h"
 #include "md5.h"
 #include "sha1.h"
@@ -41,126 +42,122 @@
 #include "sha512.h"
 #include "base64.h"
 #include "xalloc.h"
+#include "hash-pjw-bare.h"
 
 #include "utils.h"
 #include "text-options.h"
 #include "text-lines.h"
 #include "column-headers.h"
+#include "op-defs.h"
 #include "field-ops.h"
 
-/* In the future: allow users to change this */
-int field_op_output_precision = 14 ;
-
-/* Should NA/NaN/empty values be silengtly ignored? */
-extern bool remove_na_values;
-
 struct operation_data operations[] =
 {
   /* OP_COUNT */
-  {"count",   STRING_SCALAR,  IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {STRING_SCALAR,  IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_SUM */
-  {"sum",     NUMERIC_SCALAR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_SCALAR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_MIN */
-  {"min",     NUMERIC_SCALAR, AUTO_SET_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_SCALAR, AUTO_SET_FIRST, NUMERIC_RESULT},
   /* OP_MAX */
-  {"max",     NUMERIC_SCALAR, AUTO_SET_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_SCALAR, AUTO_SET_FIRST, NUMERIC_RESULT},
   /* OP_ABSMIN */
-  {"absmin",  NUMERIC_SCALAR, AUTO_SET_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_SCALAR, AUTO_SET_FIRST, NUMERIC_RESULT},
   /* OP_ABSMAX */
-  {"absmax",  NUMERIC_SCALAR, AUTO_SET_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_SCALAR, AUTO_SET_FIRST, NUMERIC_RESULT},
   /* OP_FIRST */
-  {"first",   STRING_SCALAR,  IGNORE_FIRST, GROUPING_MODE, STRING_RESULT},
+  {STRING_SCALAR,  IGNORE_FIRST, STRING_RESULT},
   /* OP_LAST */
-  {"last",    STRING_SCALAR,  IGNORE_FIRST, GROUPING_MODE, STRING_RESULT},
+  {STRING_SCALAR,  IGNORE_FIRST, STRING_RESULT},
   /* OP_RAND */
-  {"rand",    STRING_SCALAR,  IGNORE_FIRST, GROUPING_MODE, STRING_RESULT},
+  {STRING_SCALAR,  IGNORE_FIRST, STRING_RESULT},
   /* OP_MEAN */
-  {"mean",    NUMERIC_SCALAR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_SCALAR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_MEDIAN */
-  {"median",  NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_QUARTILE_1 */
-  {"q1",      NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_QUARTILE_3 */
-  {"q3",      NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_IQR */
-  {"iqr",     NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_PSTDEV */
-  {"pstdev",  NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_SSTDEV */
-  {"sstdev",  NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_PVARIANCE */
-  {"pvar",    NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_SVARIANCE */
-  {"svar",    NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_MAD */
-  {"mad",     NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_MADRAW */
-  {"madraw",  NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_S_SKEWNESS */
-  {"sskew",   NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_P_SKEWNESS */
-  {"pskew",   NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_S_EXCESS_KURTOSIS */
-  {"skurt",   NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_P_EXCESS_KURTOSIS */
-  {"pkurt",   NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_JARQUE_BETA */
-  {"jarque",  NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_DP_OMNIBUS */
-  {"dpo",     NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_MODE */
-  {"mode",    NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_ANTIMODE */
-  {"antimode",NUMERIC_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_UNIQUE */
-  {"unique",  STRING_VECTOR,  IGNORE_FIRST, GROUPING_MODE, STRING_RESULT},
+  {STRING_VECTOR,  IGNORE_FIRST, STRING_RESULT},
   /* OP_COLLAPSE */
-  {"collapse",STRING_VECTOR,  IGNORE_FIRST, GROUPING_MODE, STRING_RESULT},
+  {STRING_VECTOR,  IGNORE_FIRST, STRING_RESULT},
   /* OP_COUNT_UNIQUE */
-  {"countunique",STRING_VECTOR, IGNORE_FIRST, GROUPING_MODE, NUMERIC_RESULT},
-  /* OP_TRANSPOSE */
-  {"transpose",STRING_SCALAR, IGNORE_FIRST, TRANSPOSE_MODE, STRING_RESULT},
-  /* OP_REVERSE */
-  {"reverse", STRING_SCALAR, IGNORE_FIRST, REVERSE_FIELD_MODE, STRING_RESULT},
+  {STRING_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_BASE64 */
-  {"base64",  STRING_SCALAR, IGNORE_FIRST, LINE_MODE, STRING_RESULT},
+  {STRING_SCALAR, IGNORE_FIRST, STRING_RESULT},
   /* OP_DEBASE64 */
-  {"debase64",STRING_SCALAR, IGNORE_FIRST, LINE_MODE, STRING_RESULT},
+  {STRING_SCALAR, IGNORE_FIRST, STRING_RESULT},
   /* OP_MD5 */
-  {"md5",     STRING_SCALAR, IGNORE_FIRST, LINE_MODE, STRING_RESULT},
+  {STRING_SCALAR, IGNORE_FIRST, STRING_RESULT},
   /* OP_SHA1 */
-  {"sha1",    STRING_SCALAR, IGNORE_FIRST, LINE_MODE, STRING_RESULT},
+  {STRING_SCALAR, IGNORE_FIRST, STRING_RESULT},
   /* OP_SHA256 */
-  {"sha256",  STRING_SCALAR, IGNORE_FIRST, LINE_MODE, STRING_RESULT},
+  {STRING_SCALAR, IGNORE_FIRST, STRING_RESULT},
   /* OP_SHA512 */
-  {"sha512",  STRING_SCALAR, IGNORE_FIRST, LINE_MODE, STRING_RESULT},
-  /* OP_REMOVE_DUPS */
-  {"rmdup",   STRING_SCALAR, IGNORE_FIRST, REMOVE_DUPS_MODE, STRING_RESULT},
-  /* OP_NOOP */
-  {"noop",    STRING_SCALAR, IGNORE_FIRST, NOOP_MODE, STRING_RESULT},
-  {NULL, 0, 0, UNKNOWN_MODE, NUMERIC_RESULT}
+  {STRING_SCALAR, IGNORE_FIRST, STRING_RESULT},
+  /* OP_P_COVARIANCE */
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
+  /* OP_S_COVARIANCE */
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
+  /* OP_P_PEARSON_COR */
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
+  /* OP_S_PEARSON_COR */
+  {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
+  /* OP_BIN_BUCKETS */
+  {NUMERIC_SCALAR, IGNORE_FIRST, NUMERIC_RESULT},
+  /* OP_STRBIN */
+  {STRING_SCALAR, IGNORE_FIRST, NUMERIC_RESULT},
+  /* OP_FLOOR */
+  {NUMERIC_SCALAR, IGNORE_FIRST, NUMERIC_RESULT},
+  /* OP_CEIL */
+  {NUMERIC_SCALAR, IGNORE_FIRST, NUMERIC_RESULT},
+  /* OP_ROUND */
+  {NUMERIC_SCALAR, IGNORE_FIRST, NUMERIC_RESULT},
+  /* OP_TRUNCATE */
+  {NUMERIC_SCALAR, IGNORE_FIRST, NUMERIC_RESULT},
+  /* OP_FRACTION */
+  {NUMERIC_SCALAR, IGNORE_FIRST, NUMERIC_RESULT},
+  {0, 0, NUMERIC_RESULT}
 };
 
-const char* operation_mode_name[] = {
-  "(unknown)",   /* UNKNOWN_MODE */
-  "grouping",    /* GROUPING_MODE */
-  "transpose",   /* TRANSPOSE_MODE */
-  "line",        /* REMOVE-DUPS MODE - shown as 'line' mode */
-  "reverse",     /* REVERSE_FIELD_MODE */
-  "line",        /* LINE_MODE */
-  "no-op",       /* NO-OP MODE - for testing */
-};
-
-struct fieldop* field_ops = NULL;
+//struct fieldop* field_ops = NULL;
 
 enum { VALUES_BATCH_INCREMENT = 1024 };
 
-/* The character used to separate collapsed/uniqued strings */
-static char collapse_separator = ',';
-
-
 /* Add a numeric value to the values vector, allocating memory as needed */
-void
+static void
 field_op_add_value (struct fieldop *op, long double val)
 {
   if (op->num_values >= op->alloc_values)
@@ -183,7 +180,8 @@ field_op_reserve_out_buf (struct fieldop *op, const size_t minsize)
     }
 }
 
-void
+/* stores the hexadecimal representation of 'buffer' in op->out_buf */
+static void
 field_op_to_hex ( struct fieldop* op, const char *buffer, const size_t inlen )
 {
   static const char hex_digits[] =
@@ -248,7 +246,7 @@ field_op_replace_string (struct fieldop *op, const char* str, size_t slen)
    The returned pointer will have 'op->count+1' elements,
    pointing to 'op->count' strings + one last NULL.
 */
-const char **
+static const char **
 field_op_get_string_ptrs ( struct fieldop *op, bool sort,
                            bool sort_case_sensitive )
 {
@@ -276,119 +274,53 @@ field_op_get_string_ptrs ( struct fieldop *op, bool sort,
 }
 
 /* Sort the numeric values vector in a fieldop structure */
-void
+static void
 field_op_sort_values (struct fieldop *op)
 {
   qsortfl (op->values, op->num_values);
 }
 
-/* Converts a string to number (field number).
-   Exits with an error message (using 'op') on invalid field number. */
-static size_t
-try_get_field_number (enum operation op, const char* field_str)
+void
+field_op_init (struct fieldop* /*out*/ op,
+               enum field_operation oper,
+               bool by_name, size_t num, const char* name)
 {
-  long int val;
-  char *endptr;
-  if (strlen (field_str)==0)
-    error (EXIT_FAILURE, 0, _("invalid empty column for operation %s"),
-                               quote (operations[op].name));
-
-  errno = 0 ;
-  val = strtol (field_str, &endptr, 10);
-  /* If conversion to a number failed, assume this is a named column,
-     which will require a header line - defer error detection for later,
-     after we read the first line of the file. */
-  if (errno != 0 || endptr == field_str
-      || endptr == NULL || *endptr != 0)
-    return 0;
-
-  if (val<1)
-    error (EXIT_FAILURE, 0, _("invalid column '%s' for operation %s"),
-                               field_str,quote (operations[op].name));
-  return (size_t)val;
-}
-
-
-/* Allocate a new fieldop, initialize it based on 'oper',
-   and add it to the linked-list of operations */
-struct fieldop *
-new_field_op (enum operation oper, const char* field_name)
-{
-  struct fieldop *op = XZALLOC (struct fieldop);
+  assert (op != NULL); /* LCOV_EXCL_LINE */
+  memset (op, 0, sizeof *op);
 
   op->op = oper;
   op->acc_type = operations[oper].acc_type;
   op->res_type = operations[oper].res_type;
-  op->name = operations[oper].name;
   op->numeric = (op->acc_type == NUMERIC_SCALAR
                  || op->acc_type == NUMERIC_VECTOR);
   op->auto_first = operations[oper].auto_first;
+  op->slave = false;
+  op->slave_op = NULL;
 
-  op->field = try_get_field_number (oper, field_name);
-  if (op->field == 0)
-    {
-      op->field_by_name = true;
-      op->field_name = xstrdup (field_name);
-    }
-  else
-    {
-      op->field_by_name = false;
-      op->field_name = NULL;
-    }
+  op->field = num;
+  op->field_by_name = by_name;
+  op->field_name = (by_name)?xstrdup (name):NULL;
   op->first = true;
-  op->value = 0;
-  op->count = 0;
-
-  op->next = NULL;
-
   if (op->res_type == STRING_RESULT)
     {
       op->out_buf_alloc = 1024;
       op->out_buf = xmalloc (op->out_buf_alloc);
     }
-  op->out_buf_used = 0 ;
-
-  if (field_ops != NULL)
-    {
-      struct fieldop *p = field_ops;
-      while (p->next != NULL)
-        p = p->next;
-      p->next = op;
-    }
-  else
-    field_ops = op;
-
-  return op;
 }
 
-/* returns TRUE if any of the configured fields was using
-   a named column, therefore requiring a header line. */
-bool _GL_ATTRIBUTE_PURE
-field_op_have_named_fields ()
+/* Ensure this (master) fieldop has the same number of values as
+   as it's slave fieldop. */
+static void
+verify_slave_num_values (const struct fieldop *op)
 {
-  for (struct fieldop *p = field_ops; p ; p=p->next)
-    {
-      if (p->field_by_name)
-	return true;
-    }
-  return false;
-}
+  assert (op && !op->slave && op->slave_op);     /* LCOV_EXCL_LINE */
 
-void
-field_op_find_named_columns ()
-{
-  for (struct fieldop *p = field_ops; p ; p=p->next)
-    {
-      if (!p->field_by_name)
-        continue;
-
-      p->field = get_input_field_number (p->field_name);
-      if (p->field == 0)
-        error (EXIT_FAILURE, 0,
-                _("column name %s not found in input file"),
-                quote (p->field_name));
-      p->field_by_name = false;
-    }
+  if (op->num_values != op->slave_op->num_values)
+    error (EXIT_FAILURE, 0, _("input error for operation %s: \
+fields %"PRIuMAX",%"PRIuMAX" have different number of items"),
+                            quote (get_field_operation_name (op->op)),
+                            (uintmax_t)op->slave_op->field,
+                            (uintmax_t)op->field);
 }
 
 /* Add a value (from input) to the current field operation. */
@@ -437,7 +369,7 @@ field_op_collect (struct fieldop *op,
   if (op->first && op->auto_first && op->numeric)
       op->value = num_value;
 
-  switch (op->op)
+  switch (op->op)                                /* LCOV_EXCL_BR_LINE */
     {
     case OP_SUM:
     case OP_MEAN:
@@ -547,6 +479,10 @@ field_op_collect (struct fieldop *op,
     case OP_DP_OMNIBUS:
     case OP_MODE:
     case OP_ANTIMODE:
+    case OP_P_COVARIANCE:
+    case OP_S_COVARIANCE:
+    case OP_P_PEARSON_COR:
+    case OP_S_PEARSON_COR:
       field_op_add_value (op, num_value);
       break;
 
@@ -556,10 +492,46 @@ field_op_collect (struct fieldop *op,
       field_op_add_string (op, str, slen);
       break;
 
-    case OP_REVERSE:
-    case OP_TRANSPOSE:
-    case OP_REMOVE_DUPS:
-    case OP_NOOP:
+    case OP_BIN_BUCKETS:
+      {
+        const long double val = num_value / op->params.bin_bucket_size;
+        modfl (val, & op->value);
+        /* signbit will take care of negative-zero as well. */
+        if (signbit (op->value))
+          --op->value;
+        op->value *= op->params.bin_bucket_size;
+      }
+      break;
+
+    case OP_STRBIN:
+      op->value = hash_pjw_bare (str,slen) % (op->params.strbin_bucket_size);
+      break;
+
+    case OP_FLOOR:
+      op->value = pos_zero (floorl (num_value));
+      break;
+
+    case OP_CEIL:
+      op->value = pos_zero (ceill (num_value));
+      break;
+
+    case OP_ROUND:
+      op->value = pos_zero (roundl (num_value));
+      break;
+
+    case OP_TRUNCATE:
+      modfl (num_value, &op->value);
+      op->value = pos_zero (op->value);
+      break;
+
+    case OP_FRACTION:
+      {
+        long double dummy;
+        op->value = pos_zero (modfl (num_value, &dummy));
+      };
+      break;
+
+    case OP_INVALID:
     default:
       /* Should never happen */
       internal_error ("bad op");     /* LCOV_EXCL_LINE */
@@ -570,9 +542,9 @@ field_op_collect (struct fieldop *op,
   return rc;
 }
 
-/* Returns a nul-terimated string, composed of the unique values
-   of the input strings. The return string must be free'd. */
-void
+/* creates a list of unique strings from op->str_buf .
+   results are stored in op->out_buf. */
+static void
 unique_value ( struct fieldop *op, bool case_sensitive )
 {
   const char *last_str;
@@ -654,12 +626,16 @@ collapse_value ( struct fieldop *op )
         buf[i] = collapse_separator ;
 }
 
-void
+/* stores in op->out_buf the result of the field operation
+   when there are no input values.
+   'no values' can happen with '--narm' and input of all N/As.
+   The printed results are consistent as much as possible with R */
+static void
 field_op_summarize_empty (struct fieldop *op)
 {
   long double numeric_result = 0 ;
 
-  switch (op->op)
+  switch (op->op)                                /* LCOV_EXCL_BR_LINE */
     {
     case OP_MEAN:
     case OP_S_SKEWNESS:
@@ -680,6 +656,17 @@ field_op_summarize_empty (struct fieldop *op)
     case OP_SVARIANCE:
     case OP_MODE:
     case OP_ANTIMODE:
+    case OP_P_COVARIANCE:
+    case OP_S_COVARIANCE:
+    case OP_P_PEARSON_COR:
+    case OP_S_PEARSON_COR:
+    case OP_BIN_BUCKETS:
+    case OP_STRBIN:
+    case OP_FLOOR:
+    case OP_CEIL:
+    case OP_ROUND:
+    case OP_TRUNCATE:
+    case OP_FRACTION:
       numeric_result = nanl ("");
       break;
 
@@ -718,20 +705,18 @@ field_op_summarize_empty (struct fieldop *op)
       strcpy (op->out_buf, "");
       break;
 
-
-    case OP_TRANSPOSE: /* not handled here */
-    case OP_REVERSE:   /* not handled here */
-    case OP_REMOVE_DUPS:
-    case OP_NOOP:
+    case OP_INVALID:
     default:
       /* Should never happen */
       internal_error ("bad op");     /* LCOV_EXCL_LINE */
     }
 
   if (op->res_type==NUMERIC_RESULT)
-    printf ("%.*Lg", field_op_output_precision, numeric_result);
-  else
-    printf ("%s", op->out_buf);
+    {
+      field_op_reserve_out_buf (op, numeric_output_precision*2);
+      snprintf (op->out_buf, op->out_buf_alloc,
+                "%.*Lg", numeric_output_precision, numeric_result);
+    }
 }
 
 /* Prints to stdout the result of the field operation,
@@ -745,9 +730,12 @@ field_op_summarize (struct fieldop *op)
   /* In case of no values, each operation returns a specific result.
      'no values' can happen with '--narm' and input of all N/As. */
   if (op->count==0)
-    return field_op_summarize_empty (op);
+    {
+      field_op_summarize_empty (op);
+      return ;
+    }
 
-  switch (op->op)
+  switch (op->op)                                /* LCOV_EXCL_BR_LINE */
     {
     case OP_MEAN:
       numeric_result = op->value / op->count;
@@ -759,6 +747,13 @@ field_op_summarize (struct fieldop *op)
     case OP_MAX:
     case OP_ABSMIN:
     case OP_ABSMAX:
+    case OP_BIN_BUCKETS:
+    case OP_STRBIN:
+    case OP_FLOOR:
+    case OP_CEIL:
+    case OP_ROUND:
+    case OP_TRUNCATE:
+    case OP_FRACTION:
       /* no summarization for these operations, just print the value */
       numeric_result = op->value;
       break;
@@ -849,6 +844,28 @@ field_op_summarize (struct fieldop *op)
                                                           op->num_values );
       break;
 
+    case OP_P_COVARIANCE:
+    case OP_S_COVARIANCE:
+      assert (!op->slave);                       /* LCOV_EXCL_LINE */
+      assert (op->slave_op);                     /* LCOV_EXCL_LINE */
+      verify_slave_num_values (op);
+      numeric_result = covariance_value (op->values, op->slave_op->values,
+                                         op->num_values,
+                                         (op->op==OP_P_COVARIANCE)?
+                                                DF_POPULATION:DF_SAMPLE );
+      break;
+
+    case OP_P_PEARSON_COR:
+    case OP_S_PEARSON_COR:
+      assert (!op->slave);                       /* LCOV_EXCL_LINE */
+      assert (op->slave_op);                     /* LCOV_EXCL_LINE */
+      verify_slave_num_values (op);
+      numeric_result = pearson_corr_value (op->values, op->slave_op->values,
+                                           op->num_values,
+                                           (op->op==OP_P_PEARSON_COR)?
+                                                DF_POPULATION:DF_SAMPLE);
+      break;
+
     case OP_MODE:
     case OP_ANTIMODE:
       field_op_sort_values (op);
@@ -900,24 +917,23 @@ field_op_summarize (struct fieldop *op)
       field_op_to_hex (op, tmpbuf, 64);
       break;
 
-    case OP_TRANSPOSE: /* not handled here */
-    case OP_REVERSE:   /* not handled here */
-    case OP_REMOVE_DUPS:
-    case OP_NOOP:
+    case OP_INVALID:
     default:
       /* Should never happen */
       internal_error ("bad op");     /* LCOV_EXCL_LINE */
     }
 
   if (op->res_type==NUMERIC_RESULT)
-    printf ("%.*Lg", field_op_output_precision, numeric_result);
-  else
-    printf ("%s", op->out_buf);
+    {
+      field_op_reserve_out_buf (op, numeric_output_precision*2);
+      snprintf (op->out_buf, op->out_buf_alloc,
+                "%.*Lg", numeric_output_precision, numeric_result);
+    }
 }
 
 /* reset operation values for next group */
 void
-reset_field_op (struct fieldop *op)
+field_op_reset (struct fieldop *op)
 {
   op->first = true;
   op->count = 0 ;
@@ -928,18 +944,8 @@ reset_field_op (struct fieldop *op)
   /* note: op->str_buf and op->str_alloc are not free'd, and reused */
 }
 
-/* reset all field operations, for next group */
 void
-reset_field_ops ()
-{
-  for (struct fieldop *p = field_ops; p ; p = p->next)
-    reset_field_op (p);
-}
-
-/* Frees all memory associated with a field operation struct.
-   returns the 'next' field operation, or NULL */
-static void
-free_field_op (struct fieldop *op)
+field_op_free (struct fieldop* op)
 {
   free (op->values);
   op->num_values = 0 ;
@@ -957,107 +963,6 @@ free_field_op (struct fieldop *op)
 
   free (op->field_name);
   op->field_name = NULL;
-
-  free (op);
-}
-
-void
-free_field_ops ()
-{
-  struct fieldop *p = field_ops;
-  while (p)
-    {
-      struct fieldop *n = p->next;
-      free_field_op (p);
-      p = n;
-    }
-}
-
-/* Given a string with operation name, returns the operation enum.
-   exits with an error message if the string is not a valid/known operation. */
-enum operation
-get_operation (const char* keyword)
-{
-  for (size_t i = 0; operations[i].name ; i++)
-      if ( STREQ (operations[i].name, keyword) )
-        return (enum operation)i;
-
-  error (EXIT_FAILURE, 0, _("invalid operation '%s'"), keyword);
-  return 0; /* never reached LCOV_EXCL_LINE */
-}
-
-/* Extract the operation patterns from args START through ARGC - 1 of ARGV. */
-void
-parse_operations (enum operation_mode mode, int argc, int start, char **argv)
-{
-  int i = start; /* Index into ARGV. */
-  enum operation op;
-
-  /* From here on, by default we assume it's a "groupby" operation */
-  while ( i < argc )
-    {
-      op = get_operation (argv[i]);
-      if (operations[op].mode != mode)
-        error (EXIT_FAILURE, 0, _("conflicting operation found: "\
-               "expecting %s operations, but found %s operation %s"),
-               operation_mode_name[mode],
-               operation_mode_name[operations[op].mode],
-               quote (operations[op].name));
-
-      i++;
-      if ( i >= argc )
-        error (EXIT_FAILURE, 0, _("missing field number after " \
-                                  "operation '%s'"), argv[i-1] );
-      new_field_op (op, argv[i]);
-      i++;
-    }
-}
-
-enum operation_mode
-parse_operation_mode (int argc, int start, char** argv)
-{
-  assert (start < argc); /* LCOV_EXCL_LINE */
-
-  const enum operation op = get_operation ( argv[start] );
-  const enum operation_mode om = operations[op].mode;
-  switch (om)
-    {
-    case TRANSPOSE_MODE:
-    case REVERSE_FIELD_MODE:
-    case NOOP_MODE:
-      if ( start+1 < argc )
-        error (EXIT_FAILURE, 0, _("extra operands after '%s'"), argv[start]);
-      break;
-
-    case REMOVE_DUPS_MODE:
-    case LINE_MODE:
-    case GROUPING_MODE:
-      /* parse individual operations */
-      parse_operations (om, argc, start, argv);
-      break;
-
-    case UNKNOWN_MODE:
-    default:
-      internal_error ("unknown mode"); /* LCOV_EXCL_LINE */
-    }
-
-  return om;
-}
-
-void
-summarize_field_ops ()
-{
-  for (struct fieldop *p = field_ops; p ; p=p->next)
-    {
-      field_op_summarize (p);
-
-      /* print field separator */
-      if (p->next)
-        print_field_separator ();
-    }
-
-  /* print end-of-line */
-  print_line_separator ();
 }
 
 /* long mix function, from:
@@ -1088,17 +993,28 @@ init_random (void)
 const char*
 field_op_collect_result_name (const enum FIELD_OP_COLLECT_RESULT flocr)
 {
-  switch (flocr)
+  switch (flocr)                                 /* LCOV_EXCL_BR_LINE */
    {
    case FLOCR_INVALID_NUMBER:
      return _("invalid numeric value");
    case FLOCR_INVALID_BASE64:
      return _("invalid base64 value");
-   case FLOCR_OK:           /* LCOV_EXCL_LINE */
-   case FLOCR_OK_KEEP_LINE: /* LCOV_EXCL_LINE */
-   case FLOCR_OK_SKIPPED:   /* LCOV_EXCL_LINE */
-   default:                 /* LCOV_EXCL_LINE */
-     assert (false);        /* LCOV_EXCL_LINE */
-     return "";             /* LCOV_EXCL_LINE */
+   case FLOCR_OK:
+   case FLOCR_OK_KEEP_LINE:
+   case FLOCR_OK_SKIPPED:
+   default:
+     internal_error ("op_collect_result_name");  /* LCOV_EXCL_LINE */
+     return "";                                  /* LCOV_EXCL_LINE */
    }
 }
+
+void
+field_op_print_empty_value (enum field_operation mode)
+{
+  struct fieldop op;
+  memset (&op, 0, sizeof op);
+  op.op = mode;
+  op.res_type = NUMERIC_RESULT;
+  field_op_summarize_empty (&op);
+  fputs (op.out_buf, stdout);
+}
