@@ -1,6 +1,6 @@
 /* GNU Datamash - perform simple calculation on input data
 
-   Copyright (C) 2013-2015 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2013-2016 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -137,6 +137,64 @@ variance_value (const long double * const values, size_t n, int df)
   return variance;
 }
 
+long double _GL_ATTRIBUTE_PURE
+covariance_value ( const long double * const valuesA,
+                   const long double * const valuesB, size_t n, int df )
+{
+  long double sum=0;
+  long double meanA, meanB;
+  long double covariance;
+
+  assert (df>=0); /* LCOV_EXCL_LINE */
+  if ( (size_t)df == n )
+    return nanl ("");
+
+  meanA = arithmetic_mean_value (valuesA, n);
+  meanB = arithmetic_mean_value (valuesB, n);
+
+  sum = 0 ;
+  for (size_t i = 0; i < n; i++)
+    sum += (valuesA[i] - meanA) * (valuesB[i] - meanB);
+
+  covariance = sum / ( n - df );
+
+  return covariance;
+}
+
+long double
+pearson_corr_value ( const long double * const valuesA,
+                     const long double * const valuesB, size_t n, int df)
+{
+  long double meanA, meanB, sumA=0, sumB=0, sumCo=0;
+  long double sdA, sdB;
+  long double covariance;
+  long double cor;
+
+  assert (df>=0); /* LCOV_EXCL_LINE */
+  if ( (size_t)df == n )
+    return nanl ("");
+
+  meanA = arithmetic_mean_value (valuesA, n);
+  meanB = arithmetic_mean_value (valuesB, n);
+
+  for (size_t i = 0; i < n; i++)
+    {
+      const long double a = (valuesA[i] - meanA);
+      const long double b = (valuesB[i] - meanB);
+      sumA += a*a;
+      sumB += b*b;
+      sumCo += a*b;
+    }
+
+  covariance = sumCo/(n-df);
+  sdA = sqrtl (sumA/(n-df));
+  sdB = sqrtl (sumB/(n-df));
+
+  cor = covariance / ( sdA * sdB );
+  return cor;
+}
+
+
 long double
 stdev_value (const long double * const values, size_t n, int df)
 {
