@@ -4,7 +4,7 @@
   Tests for 'transpose' and 'reverse' operation modes.
 
 
-   Copyright (C) 2013-2015 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2013-2016 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -180,9 +180,9 @@ my @Tests =
 
     # Extra operands
     ['tr9',  'transpose aaa', {IN_PIPE=>''}, {EXIT=>1},
-      {ERR=>"$prog: extra operands after 'transpose'\n"}],
+      {ERR=>"$prog: extra operand 'aaa'\n"}],
     ['rev9', 'reverse aaa', {IN_PIPE=>''}, {EXIT=>1},
-      {ERR=>"$prog: extra operands after 'reverse'\n"}],
+      {ERR=>"$prog: extra operand 'aaa'\n"}],
 
     # empty input
     ['tr10',  'transpose', {IN_PIPE=>""}, {OUT=>""}],
